#!/usr/bin/perl -w
#-*- mode: Cperl -*-
#
##############################################################################
#
# This program converts a 'qle' SQLite log to ADIF format.
#
# Written by: Jeremy Cowgar KB8LFA
# Minor tweaks: Wilbert Knol ZL2BSJ
# Added error handling for no config file: 10/5/2006 ZL2BSJ
# Added conversion from <FREQ> in MHz to kHz: 11/8/2008 PE7T
#
##############################################################################
#
# Licence:
#
# Copyright (C) 2005 Jeremy Cowgar KB8LFA and Wilbert Knol ZL2BSJ
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

use warnings;
use strict;
use English;

my $configFile = "qle.conf";
my $adifFile   = "";
if (   defined $ARGV[0]
    && ( $ARGV[0] eq "--config" || $ARGV[0] eq "-c" )
    && defined $ARGV[1]
    && $ARGV[1] =~ /.conf$/ )
{
    $configFile = "$ARGV[1]";
    if ( defined $ARGV[2] ) { $adifFile = "$ARGV[2]"; }
}
else {
    $configFile = "qle.conf";
    if ( defined $ARGV[0] ) {
        $adifFile = "$ARGV[0]";
    }
}

print " Error: can\'t find config file $configFile\n" unless stat($configFile);
print " Error: can\'t find ADIF   file $adifFile\n"   unless stat($adifFile);

if (
    defined $ARGV[0] && ( $ARGV[0] eq "--help"
        || $ARGV[0] eq "-h" )
    || !stat($configFile)
  )
{
    print "\n";
    print " adifimport.pl attempts to open a qle config file, looks up name\n";
    print " of the database file and table name, and dumps the table in ADIF\n";
    print " format to STDOUT.\n\n";
    print " If no config file is specified, then \'qle.conf\' is assumed.\n";
    print "\n";
    print " Usage: adifimport   -h\n";
    print "                    --help\n";
    print "\n";
    print "        adifimport [ -c      <config-file>] <adif-file>\n";
    print "                   [--config <config-file>] <adif-file>\n";
    print "\n";
    print "\n";
    exit;
}

sub parse_record {
    my $cPos    = shift;
    my $content = shift;

    # my %rec = {};
    my %rec;

    my $bPos = 0;
    my $ePos = 0;

    while (( $bPos = index $content, "<", $cPos ) != -1
        && ( $ePos = index $content, ">", $bPos ) != -1 )
    {
        my ( $tname, $csize ) =
          split( /:/, substr $content, $bPos + 1, $ePos - $bPos - 1 );
        my $val = "";
        $val  = substr $content, $ePos + 1, $csize if defined $csize;
        $cPos = $ePos;
        $cPos = $ePos + $csize if defined $csize;

        if ( lc $tname eq "eor" ) { last; }
        $rec{ lc $tname } = $val;
    }

    if ( $bPos == -1 || $ePos == -1 ) {
        return ( -1, -1 );
    }

    return ( $cPos, \%rec );
}

use Config::General;
my $conf = new Config::General(
    -ConfigFile      => $configFile,
    -InterPolateVars => 1,
) or die;

my %config = $conf->getall
  or die "Could not read config file";

####
# Read the ADIF file
open( ADIF, $adifFile )
  or die "Could not open $adifFile\n";

# Eat up until the end of the header
#   TODO: make this search lower case
while (<ADIF>) {
    if (/EOH/) { last; }
}

my $content = "";
while (<ADIF>) { $content .= $_; }
close(ADIF);

####
# Parse the ADIF file

my $pos = 0;
my @recs;

while ( $pos != -1 ) {
    ( $pos, my $rec ) = parse_record( $pos, $content );
    if ( $rec == -1 ) { last; }
    push @recs, $rec;
}

####
# Build SQL statements based on parsed ADIF data

foreach my $rec (@recs) {
    my $insert = "INSERT INTO $config{tableName}";
    my $fields = "";
    my $values = "";

    foreach ( @{ $config{adifMap} } ) {
        my ( $adifName, $fname, $opts ) = split( /:/, $_ );
        my $value = $rec->{$adifName};
	$value = "" if ! defined $value;

        if ( $adifName eq "qso_date" ) {
            if ( $opts eq "yyyy" ) {
                $value = substr $value, 0, 4;
            }
            elsif ( $opts eq "mm" ) {
                $value = substr $value, 4, 2;
            }
            elsif ( $opts eq "dd" ) {
                $value = substr $value, 6, 2;
            }
        }

        if ( $adifName eq "freq" ) {
	  if ( $fname eq $config{isKhz} ) {
	    $value =~ /^(\d+)\.(\d+)$/ 
	      or die "Value $value for \<FREQ\> must be in MHz, e.g. \'14.123450\'\n";
	    die "Could not extract MHz portion from $value\n" unless defined $1;
	    die "Could not extract kHz portion from $value\n" unless defined $2;
	    my $mhz = $1;
	    my $khz = $2;
	    $value = "$1" . substr $khz, 0, 3;
	  }
	}

        if ( $adifName eq "time_on" ) {
            if ( $opts =~ /(\d{1})/ ) {
                $value = substr $value, 0, $1;
            }
	  }
        $fields .= "$fname, ";
        $values .= "'$value', ";
    }

    # TODO: not very safe doing unchecked substr bounds
    $fields = substr $fields, 0, ( length $fields ) - 2;
    $values = substr $values, 0, ( length $values ) - 2;
    my $sql = "$insert ($fields) VALUES ($values)";

    print $sql, ";\n";
}
