#!/usr/bin/perl -w
#-*- mode: Cperl -*-
#
##############################################################################
#
# This program converts a Cabrillo log to SQL statements and dumps them on
# STDOUT.
#
# Written by:  Wilbert Knol ZL2BSJ
#
##############################################################################
#
# Licence:
#
# Copyright (C) 2007 Wilbert Knol ZL2BSJ
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
#
# This Perl script reads a Cabrillo log and produces on STDOUT a list of
# SQL commands, which you can capture by re-directing them into a file.
#
# Strictly speaking, this script does not do the actual import. You will have
# to do that yourself from the sqlite3 command-line utility by loading the
# SQL dump with the command '.read <SQL-filename>'.
#
# In order for the script to work properly, you must edit the cabrilloMap
# section in your qle configuration file. Typically, this file is called
# 'qle-<version>.conf'.
#
# You may have to tweak the cabrilloMap to match the format of the particular
# Cabrillo log you are trying to import. Not all Cabrillo logs are created
# equally.
#
# It is a good idea to do a 'dry run' (without STDOUT redirection) to make
# sure the conversion takes place  without error messages.
#
##############################################################################
#
use warnings;
use strict;
use English;

my $progName = $0;
use File::Basename;
my $sprogName = basename($progName);
$progName =~ s/^\.\///;    # Zap leading './'

if ( defined $ARGV[0] && ( $ARGV[0] eq "--help" || $ARGV[0] eq "-h" ) ) {
    print "\n";
    print "Usage: cabrilloimport  -h\n";
    print "                      --help\n";
    print "\n";
    print "       cabrilloimport  -c      <config-file>\n";
    print "                      --config <config-file>\n";
    print "\n";
    exit;
}

my $configFile;
if (   defined $ARGV[0]
    && ( $ARGV[0] eq "--config" || $ARGV[0] eq "-c" )
    && ( defined $ARGV[1] && $ARGV[1] =~ /.conf$/ ) )
{
    $configFile = "$ARGV[1]";
}
else {
    $configFile = "qle.conf";
}

use Config::General;
my $conf = new Config::General(
    -ConfigFile      => "$configFile",
    -InterPolateVars => 1,
) or die;

my %config = $conf->getall
  or die "Could not read config file $configFile\n";

# Override config-file values with command-line values if present:
foreach my $k ( keys(%config) ) {
    foreach my $a (@ARGV) {
        if ( $a =~ m/--$k=(.+)/ ) {
            $config{$k} = $1;
        }
    }
}

# Read Cabrillo file, one line at a time
# and process straight away.
# Ignore all lines not starting with "QSO:"

my $cabFileName = $ARGV[0];
open( CABRILLO, $cabFileName )
  or die "Could not open $cabFileName\n";

while (<CABRILLO>) {
    if (/^QSO:/) {
        my $currentCabLine = $_;

        # Compress whitespaces into a single space:
        $currentCabLine =~ s/(\s+)/ /g;

        # ...and split on space:
        my @currentCabFields = split( / /, $currentCabLine, );

        # Example of simplified SQL statement:
        # INSERT INTO 'zl2bsj' (dd, mm, yyyy, utc, call, khz, mode)
        # VALUES ('05', '03', '2007', '0915', 'PA3DUV', '14010')
        # Build skeleton SQL INSERT statement
        my $ins = "INSERT INTO $config{tableName}";
        my $val = "VALUES (";
        my $fds = "(";

        # Scan cabrilloMap section in config file and apply mapping
        # as we build the SQL statement:
        foreach ( @{ $config{cabrilloMap} } ) {

            # Split each cabrilloMap line into parts:
            my ( $fieldNr, $fieldName, $subFieldNr ) = split( /:/, $_ );

            # Specify the cabrilloMap field names we want to INSERT:
            $fds = $fds . "$fieldName,";

            # ..and extract the corresponding VALUES from the Cabrillo log:
            if ( defined $subFieldNr ) {

                # This field consists of sub-fields (e.g. '2007-03-31'
                # ..and needs to be broken down further:
                my $a = $currentCabFields[$fieldNr];
                my @tempArry = split( /-/, $a );
                $val = $val . "\'$tempArry[$subFieldNr]\',";
            }
            else {

                # No sub-field splitting is required:
                $val = $val . "\'$currentCabFields[$fieldNr]\',";
            }
        }

        # Delete redundant trailing comma:
        $fds =~ s/,$//;
        $val =~ s/,$//;

        # ...and build SQL components into one statement:
        my $sqlStatement = "$ins " . $fds . ") " . $val . ");";

        # Now print the SQL statement on STDOUT:
        print "$sqlStatement\n";
    }
}
close(CABRILLO);

